﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CounterSoft.Gemini.Logging;
using CounterSoft.Gemini.WebServices;
using System.Text.RegularExpressions;
using CounterSoft.Gemini.Commons.Entity;

namespace CounterSoft.Gemini.SourceControl.CVSNT
{
    public class Program
    {

        public static ServiceManager GeminiServiceManager;

        static void Main(string[] args)
        {
            try
            {
                int i = 0;
                if (args != null)
                {
                    foreach (string arg in args)
                    {
                        DiagnosticsManager.TraceMessage(string.Format("Gemini CVSNT - arg #:{0} = {1}", i++, arg));
                    }
                }


                if (args == null || args.Length < 4)
                {
                    DiagnosticsManager.TraceMessage("Gemini CVSNT Not enough args.");
                    return;
                }

                if (string.Equals(args[0], "commit", StringComparison.CurrentCultureIgnoreCase))
                {
                    string repository = args[1];
                    string comment = args[2];
                    string bugIds = args[3];
                    string files = args[4];

                    List<int> bugs = new List<int>();
                    
                    if(string.IsNullOrEmpty(bugIds))
                    {
                        bugs = new List<int>(GetBugIdsFromComment(comment));
                    }
                    else
                    {
                        bugs = GetBugIdsFromBugString(bugIds);
                    }
                    
                    if(bugs.Count == 0)
                    {
                        DiagnosticsManager.TraceMessage("No Bug Ids found.");
                        return;
                    }

                    List<string> allFiles = GetFiles(files);
                    if(allFiles.Count == 0)
                    {
                        DiagnosticsManager.TraceMessage("No files found.");
                        return;
                    }

                    string path = allFiles[0];
                    
                    allFiles.RemoveAt(0);
                    if(allFiles.Count == 0)
                    {
                        DiagnosticsManager.TraceMessage("No files found.");
                        return;
                    }

                    DiagnosticsManager.TraceMessage("Constructing Service Manager.");

                    GeminiServiceManager = new ServiceManager(
                        Properties.Settings.Default.GeminiURL,
                        Properties.Settings.Default.GeminiUsername,
                        Properties.Settings.Default.GeminiPassword, "", Properties.Settings.Default.UseWindowsAuthentication);

                    DiagnosticsManager.TraceMessage("Constructed Service Manager.");

                    DiagnosticsManager.TraceMessage("Gemini Username: " + Properties.Settings.Default.GeminiUsername);

                    DiagnosticsManager.TraceMessage("Gemini Logged in user: " + GeminiServiceManager.UsersService.WhoAmI().UserName);

                    IssueCommentEN sourceControlComment = new IssueCommentEN();
                    sourceControlComment.ProjectID = 0;
                    sourceControlComment.UserID = Program.GeminiServiceManager.UsersService.WhoAmI().UserID;
                    sourceControlComment.DateCreated = DateTime.Now;

                    SourceControlEN sourceControlFile = new SourceControlEN();
                    sourceControlFile.SourceControlRepository = repository;
                    sourceControlFile.FilePath = path;

                    foreach(int issueId in bugs)
                    {
                        StringBuilder commitedFiles = new StringBuilder();
                        foreach(string file in allFiles)
                        {
                            commitedFiles.Append(file);
                            commitedFiles.Append(", ");
                            if(Properties.Settings.Default.AddIssueSourceCodeFileLink)
                            {
                                sourceControlFile.IssueID = issueId;
                                sourceControlFile.FileName = file;
                                GeminiServiceManager.IssuesService.CreateSourceControlFile(issueId, sourceControlFile);
                            }
                        }

                        if (Properties.Settings.Default.AddIssueComment)
                        {
                            sourceControlComment.IssueID = issueId;
                            sourceControlComment.ProjectID = GeminiServiceManager.IssuesService.GetIssue(issueId).ProjectID;
                            if (commitedFiles.Length > 0) commitedFiles.Remove(commitedFiles.Length - 1, 1);
                            sourceControlComment.Comment = string.Format("Comitted file(s): {0}<br>{1}", commitedFiles.ToString(), comment);
                            GeminiServiceManager.IssuesService.CreateComment(issueId, sourceControlComment);
                        }
                    }

                }
            }
            catch (Exception ex)
            {
                DiagnosticsManager.TraceException("Program", ex);
            }
        }

        internal static List<string> GetFiles(string files)
        {
            files = files.Replace(@"\ ", @"\");
            string[] fs = files.Split(' ');
            List<string> data = new List<string>();
            
            foreach(string f in fs)
            {
                data.Add(f.Replace(@"\", " "));
            }

            return data;

        }

        internal static List<int> GetBugIdsFromBugString(string bugs)
        {
            string[] ids = bugs.Split(' ');
            List<int> bugIds = new List<int>();
            foreach(string id in ids)
            {
                string s = id.TrimEnd('\\');
                int i = 0;
                if(Int32.TryParse(s, out i))
                {
                    if(i > 0 && !bugIds.Contains(i)) bugIds.Add(i);
                }
            }

            return bugIds;
        }

        internal static int[] GetBugIdsFromComment(string comment)
        {
            Regex ex = new Regex("GEM:(?<issueid>[0-9]+)", RegexOptions.IgnoreCase);

            MatchCollection matches = ex.Matches(comment);

            int[] toReturn = new int[matches.Count];

            for (int i = 0; i < matches.Count; i++)
            {
                toReturn[i] = Convert.ToInt32(matches[i].Groups["issueid"].Value);
            }

            return toReturn;
        }
    }
}
