using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using PureCM.Client;
using CounterSoft.Gemini.Commons;
using CounterSoft.Gemini.Commons.Entity;
using CounterSoft.Gemini.Logging;
using CounterSoft.Gemini.WebServices;

namespace CounterSoft.Gemini.SourceControl.PureCM
{
    class IssueIntegrator
    {
        public IssueIntegrator(Connection oConnection)
        {
            m_oConnection = oConnection;
            
            string  strURL = System.Configuration.ConfigurationSettings.AppSettings["GeminiURL"];
            string  strUser = System.Configuration.ConfigurationSettings.AppSettings["GeminiUser"];
            string  strPassword = System.Configuration.ConfigurationSettings.AppSettings["GeminiPassword"];
            string  strAPIKey = System.Configuration.ConfigurationSettings.AppSettings["GeminiAPIKey"];
            bool    bWindowsAuthentication = System.Configuration.ConfigurationSettings.AppSettings["GeminiUseWindowsAuthentication"].Length > 0;
            
            m_oGeminiServiceManager = new ServiceManager( strURL,
                                                          strUser,
                                                          strPassword,
                                                          strAPIKey,
                                                          bWindowsAuthentication );
        }

        internal void AssociateChangesetWithIssue(Changeset oChangeset, int nIssueID)
        {
            try
            {
                string strChangeItems = "";

                foreach (ChangeItem oItem in oChangeset.Items)
                {
                    if (System.Configuration.ConfigurationSettings.AppSettings["GeminiAddFileLinks"].Length > 0)
                    {
                        SourceControlEN oFileLink = new SourceControlEN();
                        oFileLink.DateCreated = DateTime.Now;
                        oFileLink.SourceControlRepository = oChangeset.Stream.StreamPath;
                        oFileLink.IssueID = nIssueID;
                        oFileLink.FileName = Path.GetFileName(oItem.Path);
                        oFileLink.FilePath = oItem.Path;

                        m_oGeminiServiceManager.IssuesService.CreateSourceControlFile(nIssueID, oFileLink);
                    }
                    
                    switch(oItem.Type)
                    {
                    case SDK.TPCMChangeItemType.pcmEdit:
                        if ( oItem.RenamePath.Length > 0)
                        {
                            strChangeItems += "Renamed '" + oItem.Path + "' to '" + oItem.RenamePath + "'<br/>";
                        }
                        else
                        {
                            strChangeItems += "Edited '" + oItem.Path + "'<br/>";
                        }
                        break;
                    case SDK.TPCMChangeItemType.pcmAdd:
                        strChangeItems += "Added '" + oItem.Path + "'<br/>";
                        break;
                    case SDK.TPCMChangeItemType.pcmDelete:
                        strChangeItems += "Deleted '" + oItem.Path + "'<br/>";
                        break;
                    }
                }

                IssueEN oGeminiIssue = m_oGeminiServiceManager.IssuesService.GetIssue(nIssueID);
                IssueCommentEN oGeminiComment = new IssueCommentEN();

                oGeminiComment.ProjectID = oGeminiIssue.ProjectID;
                oGeminiComment.IssueID = oGeminiIssue.IssueID;
                oGeminiComment.UserID = m_oGeminiServiceManager.UsersService.WhoAmI().UserID;
                oGeminiComment.Comment = GetComment(oChangeset, strChangeItems);
                oGeminiComment.DateCreated = DateTime.Now;

                m_oGeminiServiceManager.IssuesService.CreateComment(oGeminiIssue.IssueID, oGeminiComment);
            }
            catch (Exception oException)
            {
                DiagnosticsManager.TraceException("Creating comment in Gemini", oException);
            }
        }
        
        private string GetComment(Changeset oChangeset, string strChangeItems)
        {
            string strTemplatePath = System.Configuration.ConfigurationSettings.AppSettings["GeminiCommentTemplatePath"];

            if (!File.Exists(strTemplatePath))
            {
                DiagnosticsManager.TraceMessage("Template file '" + strTemplatePath + "' not found.");
                return oChangeset.Description;
            }

            StreamReader oReader = new StreamReader(strTemplatePath);
            string strTemplate = oReader.ReadToEnd();
            
            if (strTemplate.Length <= 0)
            {
                DiagnosticsManager.TraceMessage("Template file '" + strTemplatePath + "' is empty.");
                return oChangeset.Description;
            }

            strTemplate = strTemplate.Replace("$USER", oChangeset.ClientName);
            strTemplate = strTemplate.Replace("$CHANGEID", oChangeset.IdString);
            strTemplate = strTemplate.Replace("$DESCRIPTION", oChangeset.Description);
            strTemplate = strTemplate.Replace("$CHANGEITEMS", strChangeItems);

            return strTemplate;
        }

        private Connection m_oConnection;

        private ServiceManager m_oGeminiServiceManager;
    }
}
