using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using PureCM.Client;
using CounterSoft.Gemini.Logging;

namespace CounterSoft.Gemini.SourceControl.PureCM
{
    static class Program
    {
        static void Main()
        {
            try
            {
                string strServer = System.Configuration.ConfigurationSettings.AppSettings["PureCMServer"];
                string strPort = System.Configuration.ConfigurationSettings.AppSettings["PureCMPort"];
                string strUser = System.Configuration.ConfigurationSettings.AppSettings["PureCMUser"];
                string strPassword = System.Configuration.ConfigurationSettings.AppSettings["PureCMPassword"];

                Connection oConn = new Connection(strServer, System.Convert.ToInt32(strPort), SDK.TPCMAuthType.pcmauthPassword, strUser, strPassword);

                s_oIntegrator = new IssueIntegrator(oConn);

                oConn.OnChangeSubmitted = OnChangeSubmitted;

                oConn.WaitForSysEvents();
                oConn.WakeUpSysEvents();
            }
            catch (System.Exception oException)
            {
                DiagnosticsManager.TraceException("Waiting for PureCM events", oException);
            }
        }

        /// <summary>
        /// This is a called whenever a changeset is submitted in PureCM
        /// </summary>
        internal static void OnChangeSubmitted(ChangeSubmittedEvent oEvt)
        {
            System.Threading.Thread.Sleep(1000);

            Changeset oChangeset = oEvt.Changeset;

            if (oChangeset != null)
            {
                int[] anIssueIDs = ParseDescription( oChangeset.Description );
                
                foreach( int nIssueID in anIssueIDs )
                {
                    s_oIntegrator.AssociateChangesetWithIssue(oChangeset, nIssueID);
                }
            }
        }
        
        internal static int[] ParseDescription(string strDescription)
        {
            string strKey = System.Configuration.ConfigurationSettings.AppSettings["PureCMDescriptionKey"];

            Regex oExpression = new Regex(strKey + "(?<issueid>[0-9]+)", RegexOptions.IgnoreCase);

            MatchCollection oMatches = oExpression.Matches(strDescription);

            int[] anIssueIDs = new int[oMatches.Count];

            for (int i = 0; i < oMatches.Count; i++)
            {
                anIssueIDs[i] = Convert.ToInt32(oMatches[i].Groups["issueid"].Value);
            }

            return anIssueIDs;
        }
        
        static IssueIntegrator s_oIntegrator;
    }
}
