//////////////////////////////////////////////////////
// Subversion - Gemini connector application
// by: Frans Bouma
// http://www.llblgen.com
// http://weblogs.asp.net/fbouma
// 
// Special thanks to: Kognition (http://www.syringe.net.nz/PermaLink,guid,b0fa47ff-8853-4c94-b823-431f73e30ec1.aspx)
// for giving me the idea how to get started. 
//
// License: BSD2 license. 
//////////////////////////////////////////////////////
// COPYRIGHTS:
// Copyright (c)2006 Solutions Design. All rights reserved.
// 
// This source-code is released under the following license: (BSD2)
// -------------------------------------------
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met: 
//
// 1) Redistributions of source code must retain the above copyright notice, this list of 
//    conditions and the following disclaimer. 
// 2) Redistributions in binary form must reproduce the above copyright notice, this list of 
//    conditions and the following disclaimer in the documentation and/or other materials 
//    provided with the distribution. 
// 
// THIS SOFTWARE IS PROVIDED BY SOLUTIONS DESIGN ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOLUTIONS DESIGN OR CONTRIBUTORS BE LIABLE FOR 
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
// The views and conclusions contained in the software and documentation are those of the authors 
// and should not be interpreted as representing official policies, either expressed or implied, 
// of Solutions Design. 
//
//////////////////////////////////////////////////////
// Requirements: .NET 2.0, Subversion 1.x, Gemini v2.x, WSE 2.x runtime installed on server
//////////////////////////////////////////////////////
using System;
using CounterSoft.Gemini.Logging;
using CounterSoft.Gemini.WebServices;

namespace CounterSoft.Gemini.SourceControl.SVN
{
	/// <summary>
	/// Start point of the connector. The same exe is used for both pre-commit and post-commit. Based on the command line parameters, the 
	/// context is determined. 
	/// </summary>
	/// <remarks>
	/// Usage: SvnGeminiConnector action param1 param2
	/// Subversion passes 2 parameters to the pre-commit and post-commit hook executables. (see Svn docs). These are param1 and param2. 
	/// action can be: "pre-commit" or "post-commit". 
	/// </remarks>
	public class Program
	{
		#region Variables

		private enum Action
		{
			PreCommit,
			PostCommit
			// add more actions here 
		}

        public static ServiceManager GeminiServiceManager;

        #endregion

        /// <summary>
		/// Start point of the connector.
		/// </summary>
		/// <param name="args">The arguments passed on the command line.</param>
		/// <returns>0 if the connector succeeded, 1 otherwise. If 1 is returned, the error is logged to stderr.</returns>
		static int Main( string[] args )
		{
		    DiagnosticsManager.TraceMessage("Main");

			// parse command line.
			Action actionToTake = Action.PostCommit;

		    foreach (string s in args)
		    {
                DiagnosticsManager.TraceMessage("Argument: " + s);
		    }

			if( args.Length != 3 )
			{
				DiagnosticsManager.TraceMessage( string.Format("The connector was started with the wrong number of arguments: {0}. Expected: {1}. Can't continue.", args.Length, 3 ));
				return 1;
			}

			switch( args[0] )
			{
				case "pre-commit":
					actionToTake = Action.PreCommit;
					break;
				case "post-commit":
					actionToTake = Action.PostCommit;
					break;
				// add more here.
				default:
                    DiagnosticsManager.TraceMessage( string.Format("Unknown action was passed to the connector: {0}. Can't continue.", args[0]) );
					return 1;
			}

		    DiagnosticsManager.TraceMessage("Constructing Service Manager.");

            GeminiServiceManager = new ServiceManager(
                Properties.Settings.Default.GeminiURL,
                Properties.Settings.Default.GeminiUsername,
                Properties.Settings.Default.GeminiPassword, "", Properties.Settings.Default.UseWindowsAuthentication);

            DiagnosticsManager.TraceMessage("Constructed Service Manager.");

            DiagnosticsManager.TraceMessage("Gemini Username: " + Properties.Settings.Default.GeminiUsername);

            DiagnosticsManager.TraceMessage("Gemini Logged in user: " + GeminiServiceManager.UsersService.WhoAmI().UserName);

			int toReturn = 0;

			try
			{
				switch( actionToTake )
				{
					case Action.PreCommit:
						PreCommitHandler preHandler = new PreCommitHandler();
						toReturn = preHandler.Handle( args[1], args[2] );
						break;
					case Action.PostCommit:
						PostCommitHandler postHandler = new PostCommitHandler();
						toReturn = postHandler.Handle( args[1], Convert.ToInt32( args[2] ) );
						break;
				}
			}
			catch( Exception ex )
			{
				DiagnosticsManager.TraceException("Program", ex);
				toReturn = 1;
			}

            DiagnosticsManager.TraceMessage("End Main");

			return toReturn;
		}
	}
}
