//////////////////////////////////////////////////////
// Subversion - Gemini connector application
// by: Frans Bouma
// http://www.llblgen.com
// http://weblogs.asp.net/fbouma
// 
// Special thanks to: Kognition (http://www.syringe.net.nz/PermaLink,guid,b0fa47ff-8853-4c94-b823-431f73e30ec1.aspx)
// for giving me the idea how to get started. 
//
// License: BSD2 license. 
//////////////////////////////////////////////////////
// COPYRIGHTS:
// Copyright (c)2006 Solutions Design. All rights reserved.
// 
// This source-code is released under the following license: (BSD2)
// -------------------------------------------
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met: 
//
// 1) Redistributions of source code must retain the above copyright notice, this list of 
//    conditions and the following disclaimer. 
// 2) Redistributions in binary form must reproduce the above copyright notice, this list of 
//    conditions and the following disclaimer in the documentation and/or other materials 
//    provided with the distribution. 
// 
// THIS SOFTWARE IS PROVIDED BY SOLUTIONS DESIGN ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOLUTIONS DESIGN OR CONTRIBUTORS BE LIABLE FOR 
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
//
// The views and conclusions contained in the software and documentation are those of the authors 
// and should not be interpreted as representing official policies, either expressed or implied, 
// of Solutions Design. 
//
//////////////////////////////////////////////////////
// Requirements: .NET 2.0, Subversion 1.x, Gemini v2.x, WSE 2.x runtime installed on server
//////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;

namespace CounterSoft.Gemini.SourceControl.SVN
{
	/// <summary>
	/// General purpose utility class with small, static methods.
	/// </summary>
	internal static class Utilities
	{
		/// <summary>
		/// Parses the checkin comment for issueIDs. Issue IDs have to be specified in the comments as 'GEM:x' where x is the number of the issue.
		/// Example: This is the fix for GEM:123.
		/// Multiple numbers can be specified, though they all have to be prefixed with GEM:. 
		/// </summary>
        /// <param name="checkinComment">The checkin comment to parse</param>
		/// <returns>array of issueids</returns>
		internal static int[] ParseCheckinCommentForIssueIDs(string checkinComment)
		{
			Regex ex = new Regex( "GEM:(?<issueid>[0-9]+)", RegexOptions.IgnoreCase );

			MatchCollection matches = ex.Matches( checkinComment );

			int[] toReturn = new int[matches.Count];
			
            for( int i = 0; i < matches.Count; i++ )
			{
				toReturn[i] = Convert.ToInt32( matches[i].Groups["issueid"].Value );
			}

			return toReturn;
		}

        /// <summary>
        /// Parses the checkin comment for issueIDs. Issue IDs have to be specified in the comments as 'GEM:x' where x is the number of the issue.
        /// Example: This is the fix for GEM:123.
        /// Multiple numbers can be specified, though they all have to be prefixed with GEM:. 
        /// </summary>
        /// <param name="checkinComment">The checkin comment to parse</param>
        /// <returns>array of issueids</returns>
        internal static int[] ParseCheckinCommentForIssueID(string checkinComment)
        {
            Regex ex = new Regex("GEM:(?<issueid>[0-9]+)", RegexOptions.IgnoreCase);

            MatchCollection matches = ex.Matches(checkinComment);

            int[] toReturn = new int[matches.Count];

            for (int i = 0; i < matches.Count; i++)
            {
                toReturn[i] = Convert.ToInt32(matches[i].Groups["issueid"].Value);
            }

            return toReturn;
        }

        public static string GetFormattedHtmlFileList(List<CommittedFileEN> files)
        {
            string filesHtml = "";

            foreach(CommittedFileEN file in files)
            {
                filesHtml += file.GetFormattedString;
                filesHtml += "<br/>";
            }

            return filesHtml;
        }
	}

    public class CommittedFileEN
    {
        private string filePath;
        private string fileName;
        private string changeType;
        private string originalFileName;

        public string FilePath { get { return filePath; } set { filePath = value; } }
        public string FileName { get { return fileName; } set { fileName = value; } }
        public string ChangeType { get { return changeType; } set { changeType = value; } }
        public string OriginalFilename { get { return originalFileName; } set { originalFileName = value; } }
        
        public CommittedFileEN(string path, string name, string changeType, string originalFilename)
        {
            FilePath = path;
            FileName = name;
            ChangeType = changeType;
            OriginalFilename = originalFilename;
        }

        public string GetFormattedString
        {
            get { return string.Format("{0} ({1})", OriginalFilename, ChangeType); }
        }
    }
}
