// SourceSafe.h : Declaration of the CSourceSafe

#ifndef __SOURCESAFE_H_
#define __SOURCESAFE_H_

#include "resource.h"       // main symbols
#include "GeminiInternet.h"
#import "..\Lib\SSAPI.DLL" raw_interfaces_only, raw_native_types, no_namespace, named_guids 

/////////////////////////////////////////////////////////////////////////////
// CSourceSafe
class ATL_NO_VTABLE CSourceSafe : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CSourceSafe, &CLSID_SourceSafe>,
	public ISupportErrorInfo,
	public IDispatchImpl<ISourceSafe, &IID_ISourceSafe, &LIBID_GeminiVSSLib>,
	public IDispatchImpl<IVSS, &IID_IVSS, &LIBID_SourceSafeTypeLib>,
	public IVSSEvents,
	public IVSSEventHandler
{
public:
	CSourceSafe()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SOURCESAFE)
DECLARE_NOT_AGGREGATABLE(CSourceSafe)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSourceSafe)
	COM_INTERFACE_ENTRY(ISourceSafe)
//DEL 	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY2(IDispatch, ISourceSafe)
	COM_INTERFACE_ENTRY(IVSS)
	COM_INTERFACE_ENTRY(IVSSEvents)
	COM_INTERFACE_ENTRY(IVSSEventHandler)
END_COM_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// ISourceSafe
public:
// IVSS
	STDMETHOD(get_VSSDatabase)(IVSSDatabase * * ppIVSSDatabase)
	{
		if (ppIVSSDatabase == NULL)
			return E_POINTER;
			
		return E_NOTIMPL;
	}
// IVSSEvents
	STDMETHOD(BeforeAdd)(IVSSItem * pIPrj, BSTR Local, BSTR Comment, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
		
		return E_NOTIMPL;
	}
	STDMETHOD(AfterAdd)(IVSSItem * pIItem, BSTR Local, BSTR Comment)
	{
		AssociateFileWithIssues(pIItem,Comment);

		return S_OK;
	}
	STDMETHOD(BeforeCheckout)(IVSSItem * pIItem, BSTR Local, BSTR Comment, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;

		//AssociateFileWithIssues(pIItem,Comment);

        return E_NOTIMPL;
	}
	STDMETHOD(AfterCheckout)(IVSSItem * pIItem, BSTR Local, BSTR Comment)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BeforeCheckin)(IVSSItem * pIItem, BSTR Local, BSTR Comment, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
			
		//AssociateFileWithIssues(pIItem,Comment);

		return E_NOTIMPL;
	}
	STDMETHOD(AfterCheckin)(IVSSItem * pIItem, BSTR Local, BSTR Comment)
	{
        AssociateFileWithIssues(pIItem,Comment);
		
        return S_OK;
	}
	STDMETHOD(BeforeUndoCheckout)(IVSSItem * pIItem, BSTR Local, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
		
		return S_OK;
	}
	STDMETHOD(AfterUndoCheckout)(IVSSItem * pIItem, BSTR Local)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BeforeRename)(IVSSItem * pIItem, BSTR NewName, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
	
		return E_NOTIMPL;
	}
	STDMETHOD(AfterRename)(IVSSItem * pIItem, BSTR OldName)
	{
		BSTR sComment=SysAllocString(L"");
		AssociateFileWithIssues(pIItem,sComment,OldName);
		SysFreeString(sComment);

		return S_OK;
	}
	STDMETHOD(BeforeBranch)(IVSSItem * pIItem, BSTR Comment, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
			
		return E_NOTIMPL;
	}
	STDMETHOD(AfterBranch)(IVSSItem * pIItem, BSTR Comment)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BeginCommand)(LONG unused, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
			
		return E_NOTIMPL;
	}
	STDMETHOD(EndCommand)(LONG unused)
	{
		return E_NOTIMPL;
	}
	STDMETHOD(BeforeEvent)(LONG iEvent, IVSSItem * pIItem, BSTR Str, VARIANT var, VARIANT_BOOL * pbContinue)
	{
		if (pbContinue == NULL)
			return E_POINTER;
			
		return E_NOTIMPL;
	}
	STDMETHOD(AfterEvent)(LONG iEvent, IVSSItem * pIItem, BSTR Str, VARIANT var)
	{
		return E_NOTIMPL;
	}
// IVSSEventHandler
	STDMETHOD(Init)(IVSS * pIVSS)
	{
		MESSAGEBOX(NULL,"Gemini VSS - Init","Init 1",MB_OK);
		// Connect our interface so it'll receive messages.
		CComQIPtr<IConnectionPointContainer,&IID_IConnectionPointContainer> spCPC=pIVSS;


		HRESULT hr=spCPC->FindConnectionPoint(IID_IVSSEvents,&m_spCP);
		
		hr=m_spCP->Advise(reinterpret_cast<IDispatch*>(this),&m_dwConnCookie);

		IVSSDatabase *pIVssDB=NULL;
		hr=pIVSS->get_VSSDatabase(&pIVssDB);
		
		// This doesn't work!? so we take the ini file.
		BSTR b;
		hr=pIVssDB->get_DatabaseName(&b);
		SysFreeString(b);

		pIVssDB->get_SrcSafeIni(&b);
		m_sINIFile=b;
		SysFreeString(b);

		MESSAGEBOX(NULL,"Gemini VSS - Init","Init 2",MB_OK);
		return S_OK;
	}

private:
	bool AssociateFileWithIssues(IVSSItem *pIItem, BSTR sComment, BSTR sOldName=NULL);
	CComPtr<IConnectionPoint> m_spCP;
	DWORD m_dwConnCookie;
	wstring m_sINIFile;
};

#endif //__SOURCESAFE_H_
